#!/bin/bash

_BASEDIR="$(cd "${0%/*}" 2>/dev/null; echo "$PWD"/"${0##*/}" | xargs dirname)"
LOGFILE="$(basename $0).log"

RED='\033[0;31m'
GREEN='\033[0;32m'
WHITE='\033[0;37m'
NEUTRAL='\033[0m'
YELLOW='\033[0;33m'

if hash lsb_release 2>/dev/null; then
        lsb_exists=true
    else
        lsb_exists=false
fi

DISTRO_VERSION=`cat /etc/*release | grep -o "VERSION_ID.*" | awk -F"=" '{print $2}' | sed -e 's/\"//g'`
if [ -z "$DISTRO_VERSION" ]; then
	DISTRO_VERSION=`cat /etc/*release | head -n 1 | grep -oE "6"`
	elif [ -z "$DISTRO_VERSION" ]; then
		DISTRO_VERSION=`lsb_release -r | awk '{print $2}'`
fi

DISTRO_NAME=`cat /etc/*release | grep -h "^PRETTY_NAME" | awk -F"=" '{print $2}'`
if [ -z "$DISTRO_NAME" ]; then
        DISTRO_NAME=`cat /etc/*release* | head -n 1`
fi

CONF_DOWNLOAD_URL="https://nvpn.net"

printf "nVpn.net - IKEv2 setup script has detected following Version: $DISTRO_VERSION with Distro $DISTRO_NAME. During setup procedure you need to enter two details which is your nVpn Username and Password.\n\n"

echo -n "1) Enter your nVpn Username and press [ENTER]: "
read VPN_USERNAME

printf "\n"

echo -n "2) Enter your nVpn Password and press [ENTER]: "
read VPN_PASSWORD

if [ -z "$VPN_USERNAME" ]; then
	echo "Username can not be empty. Restart the script again and enter your nVpn.net Username."
	exit
fi

if [ -z "$VPN_PASSWORD" ]; then
	echo "Password can not be empty. Restart the script again and enter your nVpn.net Password."
	exit
fi

function set_google_dns
{
	# outcomment current DNS and set google DNS as new nameservers
	sed -i -e 's/nameserver/#&/g' /etc/resolv.conf && echo -e "nameserver 8.8.8.8\nnameserver 8.8.4.4" >> /etc/resolv.conf
}

function install_strongswan 
{
	# install strongswan
	if [[ "$lsb_exists" = true && $(lsb_release -i | grep -iEoc "Ubuntu") == 1 ]]; then
		STRONGSWAN_INSTALL_PATH="/etc/"
		VPN_RELOAD_CONFIG_COMMAND="ipsec restart"
		VPN_CONNECT_COMMAND="ipsec up nvpn"
                VPN_DISCONNECT_COMMAND="ipsec down nvpn"
		apt-get update -y
		apt-get install curl strongswan strongswan-plugin-eap-md5 strongswan-plugin-eap-mschapv2 -y
	elif [[ $(echo "$DISTRO_NAME" | grep -iEoc 'CentOS.*6\..*') == 1 ]]; then
		STRONGSWAN_INSTALL_PATH="/etc/strongswan/"
		VPN_RELOAD_CONFIG_COMMAND="strongswan restart"
		VPN_CONNECT_COMMAND="strongswan up nvpn"
		VPN_DISCONNECT_COMMAND="strongswan down nvpn"
		yum install epel-release -y
		yum install sudo wget curl strongswan -y
	fi
}

function paste_config
{

cat << EOF > $STRONGSWAN_INSTALL_PATH/ipsec.conf
config setup

conn nvpn
        ike=aes256-sha1-modp1024,aes128-sha1-modp1024,3des-sha1-modp1024!
        esp=aes256-sha256,aes256-sha1,3des-sha1!
        ikelifetime=60m
        keylife=20m
        rekeymargin=3m
        keyingtries=1
        keyexchange=ikev2
        leftfirewall=yes
        leftauth=eap-md5
        eap_identity=$VPN_USERNAME
        right=$hostname
        rightauth=pubkey
        rightid="C=SO, O=nVpn, CN=nvpn.so"
        rightsubnet=0.0.0.0/0
        auto=add
        leftsourceip=%config
EOF

}

function add_secrets
{
	echo "$VPN_USERNAME : EAP \"$VPN_PASSWORD\"" > $STRONGSWAN_INSTALL_PATH/ipsec.secrets
}

function add_ca
{

cat << EOF > $STRONGSWAN_INSTALL_PATH/ipsec.d/cacerts/nvpn.pem
-----BEGIN CERTIFICATE-----
MIIFKjCCAxKgAwIBAgIICFYXfl7vCFkwDQYJKoZIhvcNAQEFBQAwMzELMAkGA1UE
BhMCU08xDTALBgNVBAoTBG5WcG4xFTATBgNVBAMTDG5WcG4gUm9vdCBDQTAeFw0x
NDA5MTIxOTQ3NTRaFw0yNDA5MDkxOTQ3NTRaMDMxCzAJBgNVBAYTAlNPMQ0wCwYD
VQQKEwRuVnBuMRUwEwYDVQQDEwxuVnBuIFJvb3QgQ0EwggIiMA0GCSqGSIb3DQEB
AQUAA4ICDwAwggIKAoICAQCU1415ATbw7BmvizNskEAaqMzG1e4h7wZf+ZQRiYoK
xCFm1AX9KrFkG6k3AnQWG4C8ZGUB6z00jR0VCez7hEmhJOgDBuiqBMOzVkNj3jxa
5w48BMVzXq2XKFM/LXj8sEUA2JymFhaDFAyQwBDvwkUYOM9uFMmfkysdv/RsoZEO
LMIPWENx7sD3sGMawmMc7P0XikjIBu+QkwB70DtoyImX9b3FbisdN7t3lPm8az+T
0AsJkBo/BGHLXe/2HGFXv42bBQC3du30dXPsC58t+yhsncH03JBC9SSgpp0zKFhP
8fireya8Zbod+0/q+KvFVo0FmhSP8/09ZsC7IIx2Sgo1LBWQezUQNLMOZFjyAhmA
q093XS3+HW35UUevBKOIAsX7cY7B4ZvaYd1jnGXkoNaBxqtUGqjgHnDYfF1P7ZhN
t+x4Dw/7Dx07LgQJDBuBDAXiHLLkGLWKiFK2mef8ygFXl+g7N4qpxRJwC5KTAi4Y
clDeWyMFXzrwoUQHqNaz3h7OK+3W/mVnf06ejulNnHbgpSfwdyOyHCRrHYkQvKZB
zlVz9GJbOvr7/cmRLy4H3nZhdfxnYAm6BuRYwbEeAmf+yItvpM0P97eOcVKuBohL
PIDqXMszq/WBfsNdPtwRQUdwLSzPSEZhaWY21paTI09Ktc91MQbBcMDyvoPtRJmE
WQIDAQABo0IwQDAPBgNVHRMBAf8EBTADAQH/MA4GA1UdDwEB/wQEAwIBBjAdBgNV
HQ4EFgQU1NmrvmO5WOgLVpMhkrPjiCIK3ocwDQYJKoZIhvcNAQEFBQADggIBAGs6
adwTzbfKOEF4vNYAEB0BSiJyHzIqPJn0K9z3fUozjvQYu7VwNaYn3IDkOYvtOyAU
F2BDlk4CW6MP0M7JfjtHFOXj04lwKUBWtyKPJCPVU5Z0oTJWAiI4O0lacKTfA84d
FORdLwB375ndm4L8+CixOWqsVftxhMLKdluJiMbdO5pYisii1XlJc0RsHyw9NJfA
mgFbFZJ844RAMAgggGqVKZWYp1gwlE52YCt1PtmBc9YfLDmeUcHawIL2nRm7i+9H
ieC32xqnc+OfSrLtmT79/FBU+E/EQHgLUnaLe7VtMiMuTQjRpSXaMQgdZJKKJe/X
UWohx84E2cjyyPtdl+4SdiFlRtYR2pG76e38aRb0L98aPWoPJNpSavb1J4NvoSY2
ypvBI8kzUBslic58SSifc55m8XfaVcNrXrmJ00u40Q/+8FQ6W9HlNfciI76jYmap
PqfYBrHp3A9EETKQsdHn44QWsspz1YFM+nx0usFUMV0TYodk1ZGeKoTTL2H1i7Qy
0fTsUUo8jiAfMS0mYsD8TYg6bBazVV1mkVKwJpjE++lO99EAdsGRLfUTMKmzUJYl
CgjFNpU+Utl3fWxwcD3z/LWxXxtfUb45t/Z/x/rG38vL7xNUqZkpMNWQQNKYmbam
w58dnrWXIofk20P0kDlUbW8oXlJ303E+RX1GQsXg
-----END CERTIFICATE-----
EOF

}

function connect
{
	$VPN_RELOAD_CONFIG_COMMAND
	sleep 6
	status=`$VPN_CONNECT_COMMAND | grep -cE "established successfully"`
	if [[ $status -eq 1 ]]; then
		echo -e "${GREEN}nVpn IKEv2 Connection successfully established, your IP is: ${YELLOW}`curl -s ifconfig.co`${NEUTRAL}"
		echo -e "${WHITE}To disconnect from VPN, use the following command: ${RED}$VPN_DISCONNECT_COMMAND${NEUTRAL}"
		echo -e "${WHITE}To connect to the VPN, use the following command: ${GREEN}$VPN_CONNECT_COMMAND${NEUTRAL}"
	else
		echo -e "${RED}Error: Something went wrong here, the connect attempt has failed!${NEUTRAL}"
	fi 
}

function get_user_data
{
	response=`curl -s -X POST -F "username=$VPN_USERNAME" -F "proto=ikev2" $CONF_DOWNLOAD_URL/memberss/conf_create.php`
	if [[ $(echo $response | grep -cE "hostname=.*") -ne 0 ]]; then
		hostname=`echo $response | awk -F "=" '{ print $2 }'`
	else
		echo -e ${RED} $response ${NEUTRAL} && exit
	fi
}

set_google_dns
install_strongswan
get_user_data
paste_config
add_secrets
add_ca
connect
